#include  <stdio.h>
#include  <string.h>
#include  <time.h>

#include  <genlib.h>

#include   DUT_H
#include  "dplimits.h"
#include  "xor2.h"


/*      Alliance CAD System 2.0                                   */
/*        FITPATH Package 0.0                                     */
/*                                                                */
/*  Author  :      Jean-Paul CHAPUT                               */
/*  E-mail  :  cao-vlsi@masi.ibp.fr                               */
/* ************************************************************** */
/*  module  :  "xor2.c"                                            */


/* Module internal constants. */
#define        E_XOR2_WIDTH         2
#define        E_XOR2_SLICE         3
#define        E_XOR2_DRIVE         4
#define        E_XOR2_NOVIEW        5
#define        E_XOR2_OPENVBE       6
#define        E_XOR2_MODELNAME     7


/* Names of xor2 leaf cells. */
static char *TablLeafCellXor2[1] = {  "xr2_fp" };


/* Interface function of module.
   extern       char *Dgn_DispachXor2();
   */

/* Internal functions of module. */
static       void  Dgn_ErrorXor2();
static       void  Dgn_VhdlXor2();
static       void  Dgn_NetlistXor2();


/*  Function  :  'Dgn_DispatchXor2'                               */
/* -------------------------------------------------------------- */
/*  Interface of xor2 generator.                                  */

extern char *Dgn_DispatchXor2(xor2_modelname,
							  xor2_width    ,
							  xor2_slice    ,
							  xor2_msb0     ,
							  xor2_vhdl     ,
							  xor2_netlist  ,
							  xor2_icon     )
	char *xor2_modelname;  /* Model name.                    */
	long  xor2_width;      /* Bus wide.                      */
	long  xor2_slice;      /* LSB starting slice.            */
	long  xor2_msb0;       /* Zero as MSB index.             */
	long  xor2_vhdl;       /* Generation of behavioral view. */
	long  xor2_netlist;    /* Generation of netlist view.    */
	long  xor2_icon;       /* Generation of icon.            */
{
	static short  DispachXor2_FirstCall = FALSE;
	static  char  ModelName[C_SUPP_MBKSZNAME];

	/* The first time the generator is call : */
	/*     - Load specific errors.            */
	if (!DispachXor2_FirstCall)
	{
		Dgn_ErrorXor2();
		DispachXor2_FirstCall = TRUE;
	}

	/* Check if, at least, one view is asked. */
	if (   (xor2_netlist == FALSE)
		 &&(xor2_vhdl    == FALSE)
		 &&(xor2_icon    == FALSE) )
		Dut_PrintError( E_XOR2_ERROR, E_XOR2_NOVIEW);

	/* Check the bus wide. */
	if ((xor2_width < 1) || (xor2_width > DP_WIDTH))
		Dut_PrintError( E_XOR2_ERROR, E_XOR2_WIDTH, xor2_width);

	/* Check the LSB starting slice. */
	if ((xor2_slice < 0) || ( xor2_slice
							 +xor2_width > DP_WIDTH))
		Dut_PrintError( E_XOR2_ERROR, E_XOR2_SLICE, xor2_slice);

	/* Check the model name. */
	if (xor2_modelname == (char*)NULL)
#ifndef  CRUNCH_FILENAME
		/* Build a more explicit model name. */
		(void*)sprintf( ModelName, "xor2_%ldx%ldx%ld%c_cl",
					                   DP_WIDTH              ,
					                 xor2_width              ,
					                 xor2_slice              ,
					               ((xor2_msb0 ) ? 'm' : 'l'));
#else
		/* Build the crunched model name. */
		strcpy( ModelName, Dut_MakeCrunchModelName(
									    "od"       ,
								   8L,   DP_WIDTH-1,
								   8L, xor2_width-1,
								   8L, xor2_slice  ,
								   0L, xor2_msb0   ));
#endif
	else
		strcpy( ModelName, xor2_modelname);


	/* Generation of behaviorial view. */
	if (xor2_vhdl)
		Dgn_VhdlXor2( ModelName,
			   		  xor2_width,
					  xor2_slice,
			   		  xor2_msb0 );

	/* Generation of netlist view. */
	if (xor2_netlist)
		Dgn_NetlistXor2( ModelName,
						 xor2_width,
						 xor2_slice,
						 xor2_msb0 );

	/* Generation of icone. */
	/* if (xor2_icon) */
	
	/* Return name of the generated model. */
	return((xor2_modelname == (char*)NULL) ?      ModelName 
		                                   : xor2_modelname);
}


/*  Function  :  'Dgn_ErrorXor2'                                  */
/* -------------------------------------------------------------- */
/*  Adds generator specific errors to the main errors data base.  */

static void  Dgn_ErrorXor2()
{
	Dut_AddError( E_XOR2_ERROR, E_XOR2_WIDTH,
				  "invalid bus width : %ld.");
	Dut_AddError( E_XOR2_ERROR, E_XOR2_SLICE,
				  "invalid LSB starting slice : %ld.");
	Dut_AddError( E_XOR2_ERROR, E_XOR2_DRIVE,
				  "invalid output drive : %ld.");
	Dut_AddError( E_XOR2_ERROR, E_XOR2_MODELNAME,
				  "Missing model name.");
	Dut_AddError( E_XOR2_ERROR, E_XOR2_NOVIEW,
				  "No view to generate.");
	Dut_AddError( E_XOR2_ERROR, E_XOR2_OPENVBE,
				  "Unable to create file : \"%s\".");
}


/*  Function  :  'Dgn_NetListXor2'                                */
/* -------------------------------------------------------------- */
/*  Generation of xor2 behavioral view.                           */

static void  Dgn_VhdlXor2(xor2_modelname,
					      xor2_width    ,
					      xor2_slice    ,
						  xor2_msb0     )
	char *xor2_modelname;  /* Nom du modele.           */
	long  xor2_width;      /* Largeur du bus.          */
	long  xor2_slice;      /* LSB starting slice.      */
	long  xor2_msb0;       /* Zero comme index du MSB. */
{
	FILE *Xor2File;
	long  BitIndex,     LenString,      TmpTime;
	char  BusWide[256], TmpString[256];

	/* *** Creating the file holding behavioral view. *** */
	if ((Xor2File=mbkfopen(xor2_modelname,
						            "vbe",
						       WRITE_TEXT)) == (FILE*)NULL)
		Dut_PrintError( E_XOR2_ERROR, E_XOR2_OPENVBE,
					                  xor2_modelname);

	/* *** Write an identification header *** */

	/* Standart begin of header. */
	fprintf( Xor2File, 
"-- ###-------------------------------------------------------------###\n"
			);
	fprintf( Xor2File, 
"-- #                                                                 #\n"
			);
	fprintf( Xor2File,
"-- #    Alliance CAD System 2.0                                      #\n"
			);
    fprintf( Xor2File,
"-- #      FITPATH Package 0.0                                        #\n"
			);
    fprintf( Xor2File,
"-- #                                                                 #\n"
			);
    fprintf( Xor2File,
"-- # Author :     Jean-Paul CHAPUT                                   #\n" 
			);
    fprintf( Xor2File,
"-- # E-mail : cao-vlsi@masi.ibp.fr                                   #\n"
			);
	fprintf( Xor2File, 
"-- # *************************************************************** #\n"
			);
	fprintf( Xor2File,
"-- #                                                                 #\n"
			);
    fprintf( Xor2File,
"-- # Generator : \"xor2g\" (xor2)                                      #\n"
			);
	fprintf( Xor2File, 
"-- # --------------------------------------------------------------- #\n"
			);

	/* Print name of the generated file. */
	sprintf( TmpString, "\"%s.vbe\"."           , xor2_modelname);
    fprintf( Xor2File , "-- # File : %-55s  #\n",     TmpString);
	fprintf( Xor2File ,
"-- #                                                                 #\n"
			);

	/* Print date of generation. */
	TmpTime                = time((time_t*)NULL);
	sprintf( TmpString, "%s", asctime(localtime(&TmpTime)) );
	LenString              = strlen( TmpString);
	TmpString[LenString-1] = (char)0;
    fprintf( Xor2File , "-- #           Date : %-45s  #\n", TmpString);

	/* Give the call used to generate the model. */
	sprintf(  TmpString, "\"xor2g %ld %ld %ld%s -vhdl -o %s\"" ,
			                         DP_WIDTH                      ,
			                       xor2_width                      ,
			                       xor2_slice                      ,
			                      (xor2_msb0     ) ? " -msb0"  : "",
			                       xor2_modelname                  );
    fprintf( Xor2File , "-- # Generator Call : %-45s  #\n", TmpString);

	fprintf( Xor2File, 
"-- ###-------------------------------------------------------------###\n"
			);


	/* Select terminals (and signals) bus indexation. */
	/* (depending to the flag "xor2_msb0")            */
	if (!xor2_msb0)
		sprintf( BusWide, "(%ld downto %ld)", xor2_width-1
				                             +xor2_slice  ,
				                              xor2_slice  );
	else
		sprintf( BusWide,     "(%ld to %ld)", DP_WIDTH-xor2_slice
				                                      -xor2_width  ,
				                              DP_WIDTH-xor2_slice-1);


	/* *** Interface description. *** */
	fprintf( Xor2File, "\nENTITY %s IS\n\tPORT(\n", xor2_modelname);

	/* Terminals of data buses. */
	fprintf( Xor2File, "%14s :  in BIT_VECTOR%s;\n", "i0", BusWide);
	fprintf( Xor2File, "%14s :  in BIT_VECTOR%s;\n", "i1", BusWide);
	fprintf( Xor2File, "%14s : out BIT_VECTOR%s;\n",  "o", BusWide);

	/* Supplies terminals. */
	fprintf( Xor2File, "%14s :  in BIT;\n", "vdd");
	fprintf( Xor2File, "%14s :  in BIT\n" , "vss");

	/* *** End of interface description *** */
	fprintf( Xor2File, "\t);\nEND %s;\n\n\n", xor2_modelname);


	/* *** Behavioral view *** */
	fprintf( Xor2File, "ARCHITECTURE behaviour_data_flow OF %s IS\n\n",
			                                            xor2_modelname);
	fprintf( Xor2File, "BEGIN\n\n");
	fprintf( Xor2File, "\to%s <= (i0%s xor i1%s);\n\n" , BusWide,
			                                   	         BusWide,
			                                    	     BusWide);

	/* Check supplies connections. */
	fprintf( Xor2File, "\t-- ****************>");
	fprintf( Xor2File, " Supplies Check "      );
	fprintf( Xor2File, "<****************\n\n" );

	fprintf( Xor2File, "\tASSERT((vdd = '1') and (vss = '0'))\n");
	fprintf( Xor2File, "\t\tREPORT \"Power supply is missing on %s.\"\n",
			                                              xor2_modelname);
	fprintf( Xor2File, "\t\tSEVERITY WARNING;\n\n");

	/* *** End of behavioral view *** */
	fprintf( Xor2File, "END behaviour_data_flow;\n");

	/* *** Close the file holding the behavioral view *** */
	fclose( Xor2File);
}


/*  Function  :  'Dgn_NetListXor2'                                */
/* -------------------------------------------------------------- */
/*  Generation of xor2erter/buffer netlist view.                  */

static void  Dgn_NetlistXor2(xor2_modelname,
						     xor2_width    ,
						     xor2_slice    ,
						     xor2_msb0     )
	char *xor2_modelname;  /* Model name.            */
	long  xor2_width;      /* Bus wide.              */
	long  xor2_slice;      /* LSB starting slice.    */
	long  xor2_msb0;       /* Zero as MSB index.     */
{
	long       BusIndex;
	long  LeafCellIndex;

	char *LeafCellSliceName;
	char  LeafCellInstaName[C_SUPP_MBKSZNAME];

	char  BusNameI0[C_SUPP_MBKSZNAME];
	char  BusNameI1[C_SUPP_MBKSZNAME];
	char  BusNameO[ C_SUPP_MBKSZNAME];
	char  PinNameI0[C_SUPP_MBKSZNAME];
	char  PinNameI1[C_SUPP_MBKSZNAME];
	char  PinNameO[ C_SUPP_MBKSZNAME];


	/* Select name of the leaf cell model. */
	LeafCellSliceName = TablLeafCellXor2[0];

	/* Creation of the new logical figure. */
	DEF_LOFIG( xor2_modelname);

	/* Name of data bus terminals. */
	if (!xor2_msb0)
	{
		sprintf( BusNameI0,"i0[%ld:%ld]", xor2_width-1+xor2_slice, xor2_slice);
		sprintf( BusNameI1,"i1[%ld:%ld]", xor2_width-1+xor2_slice, xor2_slice);
		sprintf( BusNameO , "o[%ld:%ld]", xor2_width-1+xor2_slice, xor2_slice);
	}
	else
	{
		sprintf( BusNameI0,"i0[%ld:%ld]", DP_WIDTH-xor2_slice
				                                  -xor2_width  ,
				                          DP_WIDTH-xor2_slice-1);
		sprintf( BusNameI1,"i1[%ld:%ld]", DP_WIDTH-xor2_slice
				                                  -xor2_width  ,
				                          DP_WIDTH-xor2_slice-1);
		sprintf( BusNameO , "o[%ld:%ld]", DP_WIDTH-xor2_slice
				                                  -xor2_width  ,
				                          DP_WIDTH-xor2_slice-1);
	}

 
	/* Creation of figure terminals. */
	/* **** Control terminals **** */
	LOCON( BusNameI0, IN   , BusNameI0);
	LOCON( BusNameI1, IN   , BusNameI1);
	LOCON( BusNameO ,   OUT, BusNameO );

	/* **** Data bus terminals **** */
	LOCON(     "vdd", INOUT,     "vdd"); 
	LOCON(     "vss", INOUT,     "vss"); 
	
	
	/* Creation of leaf cells instances. */
	/* ("normals" cells of column)       */
	for( LeafCellIndex=xor2_slice;
		 LeafCellIndex<xor2_slice+xor2_width; LeafCellIndex++)
	{
		/* Build leaf cell name. */
		sprintf( LeafCellInstaName, "xor2_%ld", LeafCellIndex);
		
		/* Compute terminal index. */
		if (!xor2_msb0) BusIndex =              LeafCellIndex;
		else            BusIndex = DP_WIDTH -1 -LeafCellIndex;

		/* Build signals names. */
		sprintf( PinNameI0, "i0[%ld]", BusIndex);
		sprintf( PinNameI1, "i1[%ld]", BusIndex);
		sprintf( PinNameO ,  "o[%ld]", BusIndex);

		/* Leaf cell instanciation. */
		LOINS( LeafCellSliceName,
			   LeafCellInstaName,
			          PinNameI0 ,
			          PinNameI1 ,
			          PinNameO  , "vdd", "vss", EOL);
	}

	/* Ends and write to disk the new figure. */
	SAVE_LOFIG();
}
